package gps_monitor.gps;


import gps_monitor.util.*;
/**
 * This interface has been designed to help create software that
 * relies on a software 'widget', to wrap data produced by a
 * GPS (global positioning system) receiver. This is only an
 * initial attempt at specifying what elements such a widget
 * should include, and is likely to be revised in future.
 *
 * A GPS widget can be used to monitor the location of an object
 * on a global scale. This interface assumes location data is
 * represented in degrees and minutes when regarding hence latitude
 * and longitude coordinates. The data format assumed is the DDMM.MM
 * standard, not the DD.DD or DD-MM-SS standards. All values including
 * altitude must be according to the WGS84 ellipsoid.
 *	This interface relies on the GlobalCoordinate and LatOrLongitude
 * classes that are contained in the same package.
 *
 * @see	locationUtils.GlobalCoordinate
 * @see	locationUtils.LatOrLongitude
 * @author Richard Caudle
 * @version 1.0.0
 */

public interface GpsGenerator
{
	/**
	 * This method returns the mean average accuracy of the widget,
	 * which is either known as a specification of the device, or has
	 * been infered by collecting statistics on its accuracy over
	 * a length of time. The statistic is returned in metres,
	 * and regards the device's horizontal accuracy.
	 *
	 * @return The mean average accuracy of the widget/device.
	 */
	double getMeanHorizontalAccuracy() throws NullPointerException;

	/**
	 * This method returns the worst-case accuracy of the widget
	 * which is usually a known property of the device that will
	 * remain static. The statistic will be returned in metres,
	 * and regards the device's horizontal accuracy.
	 *
	 * @return The worst-case average accuracy of the widget/device.
	 */
	double getWorstCaseHorizontalAccuracy() throws NullPointerException;

	/**
	 * This method returns the time at which the Gps last calculated
	 * its position.
	 * There are occasions when we would like to know at what time the last
	 * location data was received. For example, this allows us to find when we last knew
	 * the object's position and hence whether the location data is still of
	 * high enough quality.
	 *
	 * The time will be according to UTC. This time format enumerates the number
	 * of milliseconds that have elapsed since midnight GMT, January 1st, 1970.
     *
	 * @return The time at which the widget last knew the object's position.
	 */
	long getTimeLastLocationUpdate() throws NullPointerException;

	/**
	 * This method returns the current longitude of the object,
	 * as a LatOrLongitude object.
	 * It is important to state at this point that widgets using this
	 * interface should regard positive numbers represent longitude given
	 * in the East direction, and negative in the West.
	 *
	 * @return The objects position regarding its longitude
	 */
	LatOrLongitude getLongitude() throws NullPointerException;

	/**
	 * This method returns the current latitude of the object,
	 * as a LatOrLongitude object. It is important to state at
	 * this point that widgets using this
	 * interface should regard positive numbers represent latitude given
	 * in the North direction, and negative in the South.
	 *
	 * @return The objects position regarding its latitude
	 */
	LatOrLongitude getLatitude() throws NullPointerException;

	/**
	 * The altitude of an object is the distance it is above (or below) sea-
	 * level. This requires different units to that of longitude and latitude
	 * and this interface assumes the use of metres. Clearly if the returned
	 * double is negative then the object is below sea-level.
	 *
	 * @return The objects position regarding its altitude
	 */
	double getAltitude() throws NullPointerException;

	/**
	 * This method allows us to retreive the 3D position of the gps
	 * recevier, ie its latitude, longitude and altitude.  It returns
	 * the result as a GlobalCoordinate.
	 *
	 * @return The receiver's 3D position
	 */
	GlobalCoordinate getLocation() throws NullPointerException;

	/**
	 * To obtain the heading on which the object is moving this method
	 * should be used. Note the heading is the direction in which the object
	 * is travelling, not the bearing which is the direction to a destination/
	 * waypoint.  The heading is measured in degrees and is relative to True
	 * North.
	 *
	 * @return The true heading on which the object is travelling
	 */
	double getTrueHeading() throws NullPointerException;

	/**
	 * To obtain the speed at which the object travelling this method
	 * should be used. The returned value will be in kilometres/hour.
	 *
	 * @return The latest known speed of object
	 */
	double getSpeed() throws NullPointerException;

	/**
	 * Calling this method returns a string describing the current status
	 * of Gps generator. There are likely to be four alternatives "active",
	 * "inactive", "reception_warning" and "out_of_range".
	 * "reception_warning" refers to a status when the receiver/generator will
	 * still produce location data, but is not confident of the data's accuracy.
	 *
	 * @return The current status of the widget/device
	 */
	String getStatus() throws NullPointerException;
}